# teensylogger.py
# teensylogger python module

# www.weigu.lu/b/teensylogger

import serial
from time import strftime, localtime, strptime      # sleep, mktime, gmtime
from datetime import datetime, timedelta
#import shutil
#import os
#import shlex


class Teensylogger:
    def __init__(self, port, speed, timeout, imp_per_kWh):
        self.port = port
        self.speed = speed
        self.timeout = timeout
        self.imp_per_kWh = imp_per_kWh
        self.time_Ymd = strftime("%Y_%m_%d", localtime())
        self.time_dmy = strftime("%d.%m.%y", localtime())

        if self.imp_per_kWh == 1000:
            self.p_fact = 60
        elif self.imp_per_kWh == 2000:
            self.p_fact = 30
        else:
            print 'error imp/kWh not declared'
    # -------------------------------------------------------------------------
    # Scans for available serial ports (from jcf)
    # -------------------------------------------------------------------------

    def scan_serial(self):
        portnames = []
        for i in range(50):            # Linux ttyS
            try:
                name = "/dev/ttyS" + str(i)
                s = serial.Serial(name)
                s.close()
                portnames.append(name)
            except:
                pass
        for i in range(50):            # Linux USB
            try:
                name = "/dev/ttyUSB" + str(i)
                s = serial.Serial(name)
                s.close()

                portnames.append(name)
            except:
                pass
        for i in range(50):            # Windows
            try:
                name = "COM" + str(i)
                s = serial.Serial(name)
                s.close()
                portnames.append(name)
            except:
                pass
        return portnames

    # -------------------------------------------------------------------------
    # return serial minute data from teensylogger in cooked format
    # save the raw data and cooked data in a file
    # -------------------------------------------------------------------------
    def get_min(self, datafile_raw, datafile_cooked):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            datalist = ['Cannot open serial port']
            return datalist
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          # 7840 (dropping bytes in Linux)
            i += 1
            ser.write('g\r')
            print "Data send: 'g' + CR"
            for j in range(41):        # 41 Byte
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        datalist = data.split()
        print datalist
        if datalist == []:
            datalist = ['no data! wrong port?']
            return datalist
        for i in range(2, 10):
            datalist[i] = str(int(datalist[i], 16) * self.p_fact)
        print datalist
        global ftime
        try:
            f = open(datafile_raw + self.time_Ymd + '.min', 'a+')  # app.+read
        except IOError:
            print ('Cannot create or find file: ' + datafile_raw)
        try:
            f2 = open(datafile_cooked + self.time_Ymd + '.min', 'a')
        except IOError:
            print ('Cannot create or find file: ' + datafile_cooked)
        lineList = f.readlines()            # read all lines
        if (len(lineList)) == 0:
            line_data = 'date time P1[W] P2[W] P3[W] P4[W] P5[W] P6[W] ' \
                + 'P7[W] P8[W]' + '\n' + ' '.join(datalist) + '\n'
        else:
            line_data = ' '.join(datalist) + '\n'
        f.write(line_data)
        f2.write(data + '\n')
        f.close()
        f2.close()
        return datalist

    # -------------------------------------------------------------------------
    # return serial quarter data from teensylogger in cooked format
    # save the raw data and cooked data in a file
    # -------------------------------------------------------------------------
    def get_qua(self, datafile_raw, datafile_cooked):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('q\r')
            print "Data send: 'q' + CR"
            for j in range(356):   # 16 (date) + 7*16*3 (xx_) + \r + 3 (OK\r)
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        print data
        if data == '':
            datalist = ['no data! wrong port?']
            return datalist
        print data[0:14]
        datalist = []
        d = datetime.strptime(data[0:14], "%d.%m.%y %H:%M")
        for j in range(7):
            d = d + timedelta(seconds=900)
            mdate = str("%02d.%02d.%s" % (d.day, d.month, d.year))
            datalist.append(mdate[0:6] + mdate[8:10])
            datalist.append("%02d:%02d:00" % (d.hour, d.minute))
            for i in range(8):
                num = (int(data[j * 48 + i * 6 + 16:j * 48 + i * 6 + 18],
                       16) * 256 + int(data[j * 48 + i * 6 + 19:j * 48 +
                       i * 6 + 21], 16))
                if self.imp_per_kWh == 2000:
                    num = num / 2
                if self.imp_per_kWh != 2000 and self.imp_per_kWh != 1000:
                    print 'error imp/kWh not declared'
                if i == 7:
                    datalist.append(str(num) + '\n')
                else:
                    datalist.append(str(num))
        print datalist
        global ftime
        try:
            f = open(datafile_raw + self.time_Ymd + '.qua', 'a+')  # app.+read
        except IOError:
            print ('Cannot create or find file: ' + datafile_raw)
        try:
            f2 = open(datafile_cooked + self.time_Ymd + '.qua', 'a')
        except IOError:
            print ('Cannot create or find file: ' + datafile_cooked)
        lineList = f.readlines()            # read all lines
        if (len(lineList)) == 0:
            line_data = 'date time E1[Wh] E2[Wh] E3[Wh] E4[Wh] E5[Wh]' \
                        + 'E6[Wh] E7[Wh] E8[Wh]\n' + ' '.join(datalist)
        else:
            line_data = ' '.join(datalist) + '\n'
        f.write(line_data)
        f2.write(data[0:352] + '\n')
        f.close()
        f2.close()
        return datalist

    # -------------------------------------------------------------------------
    # return serial day data from teensylogger in cooked format
    # save the raw data and cooked data in a file
    # -------------------------------------------------------------------------
    def get_day(self, datafile_raw, datafile_cooked):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('d\r')
            print "Data send: 'd' + CR"
            for j in range(356):   # 16 (date) + 7*16*3 (xx_) + \r +  3 (OK\r)
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        print data
        if data == '':
            datalist = ['no data! wrong port? try again!']
            return datalist
        print data[0:14]
        datalist = []
        d = datetime.strptime(data[0:14], "%d.%m.%y %H:%M")
        for j in range(7):
            d = d + timedelta(days=1)
            mdate = str("%02d.%02d.%s" % (d.day, d.month, d.year))
            datalist.append(mdate[0:6] + mdate[8:10])
            datalist.append("%02d:%02d:00" % (d.hour, d.minute))
            for i in range(8):
                num = (int(data[j * 48 + i * 6 + 16:j * 48 + i * 6 + 18],
                       16) * 256 + int(data[j * 48 + i * 6 + 19:j * 48 +
                       i * 6 + 21], 16))
                if self.imp_per_kWh == 2000:
                    num = num / 2
                if self.imp_per_kWh != 2000 and self.imp_per_kWh != 1000:
                    print 'error imp/kWh not declared'
                if i == 7:
                    datalist.append(str(num) + '\n')
                else:
                    datalist.append(str(num))
        print datalist
        global ftime
        try:
            f = open(datafile_raw + self.time_Ymd + '.day', 'a+')  # app.+read
        except IOError:
            print ('Cannot create or find file: ' + datafile_raw)
        try:
            f2 = open(datafile_cooked + self.time_Ymd + '.day', 'a')
        except IOError:
            print ('Cannot create or find file: ' + datafile_cooked)
        lineList = f.readlines()            # read all lines
        if (len(lineList)) == 0:
            line_data = 'date time E1[Wh] E2[Wh] E3[Wh] E4[Wh] E5[Wh]' \
                        + 'E6[Wh] E7[Wh] E8[Wh]\n' + ' '.join(datalist)
        else:
            line_data = ' '.join(datalist) + '\n'
        f.write(line_data)
        f2.write(data[0:352] + '\n')
        f.close()
        f2.close()
        return datalist

    # -------------------------------------------------------------------------
    # return all quarter data from teensylogger in cooked format
    # save the raw data and cooked data in a file
    # -------------------------------------------------------------------------
    def get_all_qua(self, datafile_raw, datafile_cooked):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('aq\r')
            print "Data send: 'aq' + CR"
            for j in range(158147):     # 448pages * 353 byte/page + OK\r
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        print data
        if data == '':
            datalist = ['no data! wrong port? try again!']
            return datalist
        datalist = []
        for k in range(448):
            try:
                d = datetime.strptime(data[k * 353 + 0:k * 353 + 14],
                                      "%d.%m.%y %H:%M")
                for j in range(7):
                    d = d + timedelta(seconds=900)
                    mdate = str("%02d.%02d.%s" % (d.day, d.month, d.year))
                    datalist.append(mdate[0:6] + mdate[8:10])
                    datalist.append("%02d:%02d:00" % (d.hour, d.minute))
                    for i in range(8):
                        num = (int(data[k * 353 + j * 48 + i * 6 + 16:k * 353
                                        + j * 48 + i * 6 + 18], 16) * 256 +
                               int(data[k * 353 + j * 48 + i * 6 + 19:k * 353
                                        + j * 48 + i * 6 + 21], 16))
                        if self.imp_per_kWh == 2000:
                            num = num / 2
                        if self.imp_per_kWh != 2000 and \
                           self.imp_per_kWh != 1000:
                            print 'error imp/kWh not declared'
                        if i == 7:
                            datalist.append(str(num) + '\n')
                        else:
                            datalist.append(str(num))
            except:
                pass
        if datalist == []:
            datalist = ['no data! wrong port? try again!']
        #print datalist
        global ftime
        try:
            f = open(datafile_raw + self.time_Ymd + '_all.qua', 'a+')  # a+read
        except IOError:
            print ('Cannot create or find file: ' + datafile_raw)
        try:
            f2 = open(datafile_cooked + self.time_Ymd + '_all.qua', 'a')
        except IOError:
            print ('Cannot create or find file: ' + datafile_cooked)
        lineList = f.readlines()            # read all lines
        if (len(lineList)) == 0:
            line_data = 'date time E1[Wh] E2[Wh] E3[Wh] E4[Wh] E5[Wh]' \
                        + 'E6[Wh] E7[Wh] E8[Wh]\n' + ' '.join(datalist) + '\n'
        else:
            line_data = ' '.join(datalist) + '\n'
        f.write(line_data)
        f2.write(data[0:158144] + '\n')
        f.close()
        f2.close()
        return datalist

    # -------------------------------------------------------------------------
    # return all day data from teensylogger in cooked format
    # save the raw data and cooked data in a file
    # -------------------------------------------------------------------------
    def get_all_day(self, datafile_raw, datafile_cooked):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('ad\r')
            print "Data send: 'ad' + CR"
            for j in range(22595):        # 64pages * 353 byte/page + OK\r
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        print data
        if data == '':
            datalist = ['no data! wrong port? try again!']
            return datalist
        datalist = []
        for k in range(64):
            try:
                d = datetime.strptime(data[k * 353 + 0:k * 353 + 14],
                                      "%d.%m.%y %H:%M")
                for j in range(7):
                    d = d + timedelta(days=1)
                    mdate = str("%02d.%02d.%s" % (d.day, d.month, d.year))
                    datalist.append(mdate[0:6] + mdate[8:10])
                    datalist.append("%02d:%02d:00" % (d.hour, d.minute))
                    for i in range(8):
                        num = (int(data[k * 353 + j * 48 + i * 6 + 16:k * 353
                                        + j * 48 + i * 6 + 18], 16) * 256 +
                               int(data[k * 353 + j * 48 + i * 6 + 19:k * 353
                                        + j * 48 + i * 6 + 21], 16))
                        if self.imp_per_kWh == 2000:
                            num = num / 2
                        if self.imp_per_kWh != 2000 and \
                           self.imp_per_kWh != 1000:
                            print 'error imp/kWh not declared'
                        if i == 7:
                            datalist.append(str(num) + '\n')
                        else:
                            datalist.append(str(num))
            except:
                break
        if datalist == []:
            datalist = ['no data! wrong port? try again!']
        #print datalist
        global ftime
        try:
            f = open(datafile_raw + self.time_Ymd + '_all.day', 'a+')  # a+read
        except IOError:
            print ('Cannot create or find file: ' + datafile_raw)
        try:
            f2 = open(datafile_cooked + self.time_Ymd + '_all.day', 'a')
        except IOError:
            print ('Cannot create or find file: ' + datafile_cooked)
        lineList = f.readlines()            # read all lines
        if (len(lineList)) == 0:
            line_data = 'date time E1[Wh] E2[Wh] E3[Wh] E4[Wh] E5[Wh]' \
                        + 'E6[Wh] E7[Wh] E8[Wh]\n' + ' '.join(datalist) + '\n'
        else:
            line_data = ' '.join(datalist) + '\n'
        f.write(line_data)
        f2.write(data[0:22592] + '\n')
        f.close()
        f2.close()
        return datalist

   # -------------------------------------------------------------------------
    # get_pointers
    # -------------------------------------------------------------------------
    def get_pointers(self):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('p\r')
            print 'Data send: p + CR'
            for j in range(16):
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        if data == '':
            datalist = ['no return data! wrong port? try again!']
            return datalist
        print data
        datalist = []
        datalist.append('0x' + data[0:2] + data[3:5])
        datalist.append('0x' + data[6:8] + data[9:11])
        print datalist
        return datalist

    # -------------------------------------------------------------------------
    # set_pointers
    # -------------------------------------------------------------------------
    def set_pointers(self, p_text):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('s ' + p_text + '\r')
            print 'Data send: s ' + p_text + '  + CR'
            for j in range(3):
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        if data == '':
            datalist = ['no return data! wrong port? try again!']
            return datalist
        print data
        datalist = []
        if data == 'OK\r':
            datalist.append('The pointer addresses are set to the new values!')
        print datalist
        return datalist

    # -------------------------------------------------------------------------
    # clear all data and reset pointer
    # -------------------------------------------------------------------------
    def clear_data(self):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('c\r')
            print "Data send: 'c' + CR"
            for j in range(527):    # 11 Byte + 512 byte + +\r  + 3 (OK\r)
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        print data
        datalist = []
        datalist.append('All data cleared and pointers reset (0000E000)')
        print datalist
        return datalist

    # -------------------------------------------------------------------------
    # set_date_time
    # -------------------------------------------------------------------------
    def set_date_time(self, dt_text):
        try:
            ser = serial.Serial(self.port, self.speed, timeout=self.timeout)
        except:
            print ('Cannot open serial port')
        data = ''
        i = 0
        while len(data) == 0 and i < 50:  # needed for PL2303-H and Moschip
                                          #  7840 under Linux (dropping bytes)
            i += 1
            ser.write('t ' + dt_text + '\r')
            print 'Data send: t ' + dt_text + '  + CR'
            for j in range(3):
                try:
                    char = ser.read()
                    data += char
                    if len(data) == 0:
                        break
                except:
                    data = 'error in serial communication!'
        ser.close()             # close serial port
        print 'Connection closed!'
        if data == '':
            datalist = ['no return data! wrong port? try again!']
            return datalist
        print data
        datalist = []
        if data == 'OK\r':
            datalist.append('The date and time are set to the new values!')
        print datalist
        return datalist
